@@ -424,6 +424,7 @@ main_window_error_display (EmpathyMainWindow *window,
 	label = gtk_label_new (str);
 	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
 	gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 	gtk_widget_show (label);
 	g_free (str);
 
@@ -711,7 +712,7 @@ main_window_view_show_offline_cb (GtkToggleAction   *action,
 
 	current = gtk_toggle_action_get_active (action);
 	empathy_conf_set_bool (empathy_conf_get (),
-			      EMPATHY_PREFS_CONTACTS_SHOW_OFFLINE,
+			      EMPATHY_PREFS_UI_SHOW_OFFLINE,
 			      current);
 
 	/* Turn off sound just while we alter the contact list. */
@@ -1483,10 +1484,10 @@ empathy_main_window_show (void)
 
 	/* Show offline ? */
 	empathy_conf_get_bool (conf,
-			      EMPATHY_PREFS_CONTACTS_SHOW_OFFLINE,
+			      EMPATHY_PREFS_UI_SHOW_OFFLINE,
 			      &show_offline);
 	empathy_conf_notify_add (conf,
-				EMPATHY_PREFS_CONTACTS_SHOW_OFFLINE,
+				EMPATHY_PREFS_UI_SHOW_OFFLINE,
 				main_window_notify_show_offline_cb,
 				show_offline_widget);
 
